/********************************************************************** 
 Freeciv - Copyright (C) 2009 - Andreas Røsdal   andrearo@pvv.ntnu.no
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/


var mouse_x;
var mouse_y;
var prev_mouse_x;
var prev_mouse_y;
var keyboard_input = true;
var unitpanel_active = false;
var allow_right_click = false;

var current_focus = [];
var goto_active = false;
var paradrop_active = false;
var airlift_active = false;

/* Selecting unit from a stack without popup. */
var SELECT_POPUP = 0;
var SELECT_SEA = 1;
var SELECT_LAND = 2;
var SELECT_APPEND = 3;

var intro_click_description = true; 

var goto_request_map = {};
var goto_turns_request_map = {};
var current_goto_turns = 0;
var waiting_units_list = [];
var show_citybar = true;

/****************************************************************************
...
****************************************************************************/
function control_init() 
{
  // Register keyboard and mouse listener using JQuery.
  $(document).keydown (keyboard_listener);
  $("#canvas").mouseup(mapview_mouse_click);
  $("#canvas").mousedown(mapview_mouse_down);
  $(window).mousemove(mouse_moved_cb);
  $(window).resize(mapview_window_resized);
  $(window).bind('orientationchange resize', orientation_changed);

  if (is_touch_device()) {
    $('#canvas').bind('touchstart', mapview_touch_start);
    $('#canvas').bind('touchend', mapview_touch_end);
    $('#canvas').bind('touchmove', mapview_touch_move);
  }
  
  $("#turn_done_button").click(send_end_turn);
  if (!is_touch_device()) $("#turn_done_button").tooltip();

  $("#freeciv_logo").click(function(event) {
    window.open('/', '_new');
    });


  $("#game_text_input").keydown(function(event) {
	  return check_text_input(event, $("#game_text_input"));
    });
  $("#game_text_input").focus(function(event) {
	keyboard_input=false;
    });

  $("#game_text_input").blur(function(event) {
	  keyboard_input=true;
    });

  /* disable text-selection, as this gives wrong mouse cursor 
   * during drag to goto units. */
  document.onselectstart = function(){ return false; }

  /* disable right clicks. */
  document.oncontextmenu = function(){return allow_right_click;};

  $.contextMenu({
        selector: '#canvas', 
	zIndex: 5000,
        autoHide: true,
        callback: function(key, options) {
	  handle_context_menu_callback(key);
        },
        build: function($trigger, e) {
            var unit_actions = update_unit_order_commands();
            return {
                 callback: function(key, options) {
                   handle_context_menu_callback(key);
                  } ,
                 items: unit_actions
            };
        }
  });

 
  $(window).on('unload', function(){
    send_surrender_game();
    network_stop();
  });

  /* Click callbacks for main tabs. */
  $("#map_tab").click(function(event) {
    set_default_mapview_active();
  });

  $("#civ_tab").click(function(event) {
    set_default_mapview_inactive();
    init_civ_dialog();
  });

  $("#tech_tab").click(function(event) {
    set_default_mapview_inactive(); 
    update_tech_screen();
  });

  $("#players_tab").click(function(event) {
    set_default_mapview_inactive(); 
    update_nation_screen();
  });

  $("#opt_tab").click(function(event) {
    $("#tabs-hel").hide();
    init_options_dialog(); 
    set_default_mapview_inactive();
  });

  $("#chat_tab").click(function(event) {
    chatbox_resized();
    set_default_mapview_inactive();
    $("#tabs-chat").show();

  });


  $("#hel_tab").click(function(event) {
    set_default_mapview_inactive();
    show_help();
  });

  if (!is_touch_device()) { 
    $("#game_unit_orders_default").tooltip();
  }

  $("#game_overview_map").click(function(e) {
    var x = e.pageX - $(this).offset().left;
    var y = e.pageY - $(this).offset().top;
    overview_clicked (x, y)
  });
    
}

/****************************************************************************
 determined if this is a touch enabled device, such as iPhone, iPad.
****************************************************************************/
function is_touch_device() {
  if(('ontouchstart' in window) || 'onmsgesturechange' in window 
      || window.DocumentTouch && document instanceof DocumentTouch) {    
    return true;
  } else {
    return false;
  }
}

/****************************************************************************
...
****************************************************************************/
function mouse_moved_cb(e)
{
  mouse_x = 0;
  mouse_y = 0;
  if (!e) {
    var e = window.event;
  }
  if (e.pageX || e.pageY) {
    mouse_x = e.pageX;
    mouse_y = e.pageY;
  } else {
    if (e.clientX || e.clientY) {
      mouse_x = e.clientX;
      mouse_y = e.clientY;
    }
  }
  if (active_city == null && mapview_canvas != null 
      && $("#canvas").length) {
    mouse_x = mouse_x - $("#canvas").offset().left;
    mouse_y = mouse_y - $("#canvas").offset().top;
  } else if (active_city != null && city_canvas != null 
             && $("#city_canvas").length) {
    mouse_x = mouse_x - $("#city_canvas").offset().left;
    mouse_y = mouse_y - $("#city_canvas").offset().top;

  }

  if (client.conn.playing == null) return;

  if (C_S_RUNNING == client_state()) {
    update_mouse_cursor();
  }

}

/****************************************************************************
...
****************************************************************************/
function update_mouse_cursor()
{
  if (tech_dialog_active) {
    update_tech_dialog_cursor();
    return;
  }
      
  var ptile = canvas_pos_to_tile(mouse_x, mouse_y);

  if (ptile == null) return;

  var punit = find_visible_unit(ptile);
  var pcity = tile_city(ptile);

  if (goto_active && current_goto_turns != null) {
    /* show goto cursor */
    mapview_canvas.style.cursor = "crosshair";
  } else if (goto_active && current_goto_turns == null) {
    /* show invalid goto cursor*/
    mapview_canvas.style.cursor = "not-allowed";
  } else if (pcity != null && city_owner_player_id(pcity) == client.conn.playing.playerno) { 
    /* selecti city cursor*/
    mapview_canvas.style.cursor = "pointer";
  } else if (punit != null && punit['owner'] == client.conn.playing.playerno) {
    /* move unit cursor */
    mapview_canvas.style.cursor = "move";
  } else {
    mapview_canvas.style.cursor = "default";
  }

}

/****************************************************************************
...
****************************************************************************/
function check_text_input(event,chatboxtextarea) {
	 
  if (event.keyCode == 13 && event.shiftKey == 0)  {
    var message = $(chatboxtextarea).val();
    message = message.replace(/^\s+|\s+$/g,"");
    message = message.replace("'", "");
    message = encodeURIComponent(message);

    $(chatboxtextarea).val('');
    $(chatboxtextarea).focus();
    keyboard_input = true;
    var test_packet = {"pid" : packet_chat_msg_req, "message" : message};
    var myJSONText = JSON.stringify(test_packet);
    send_request(myJSONText);
    return false;
  }
}



/****************************************************************************
  Return TRUE iff a unit on this tile is in focus.
****************************************************************************/
function get_focus_unit_on_tile(ptile)
{

  var funits = get_units_in_focus();
  if (funits == null) return null;
  
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    if (punit['tile'] == ptile['index']) {
      return punit;
    } 
  } 
  return null;
}


/****************************************************************************
  Return TRUE iff this unit is in focus.
  TODO: not implemented yet.
****************************************************************************/
function unit_is_in_focus(cunit)
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    if (punit['id'] == cunit['id']) {
      return true;
    } 
  } 
  return false;
}

/****************************************************************************
  Returns a list of units in focus.
****************************************************************************/
function get_units_in_focus()
{
  return current_focus;
}

/**************************************************************************
 If there is no unit currently in focus, or if the current unit in
 focus should not be in focus, then get a new focus unit.
 We let GOTO-ing units stay in focus, so that if they have moves left
 at the end of the goto, then they are still in focus.
**************************************************************************/
function update_unit_focus()
{
  if (active_city != null) return; /* don't change focus while city dialog is active.*/

  /* iterate zero times for no units in focus,
   * otherwise quit for any of the conditions. */
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 

    if (punit['movesleft'] > 0 
	  && punit['done_moving'] == false
	  && punit['ai'] == false
	  && punit['activity'] == ACTIVITY_IDLE) {
      return;
    }

  }
  
  advance_unit_focus();

}

/**************************************************************************
 This function may be called from packhand.c, via update_unit_focus(),
 as a result of packets indicating change in activity for a unit. Also
 called when user press the "Wait" command.
 
 FIXME: Add feature to focus only units of a certain category.
**************************************************************************/
function advance_unit_focus()
{
  if (client_is_observer()) return;
  
  var funits = get_units_in_focus();

  var candidate = find_best_focus_candidate(false);
  
  if (candidate == null) {
    candidate = find_best_focus_candidate(true);
  }
  
  if (candidate != null) {
    set_unit_focus_and_redraw(candidate);
  } else {
    /* Couldn't center on a unit, then try to center on a city... */
    current_focus = []; /* Reset focus units. */
    update_unit_info_label(current_focus);
    $("#game_unit_orders_default").hide();
    /* find a city to focus on. */
    for (city_id in cities) {
      var pcity = cities[city_id];
      if (city_owner_player_id(pcity) == client.conn.playing.playerno) {
        center_tile_mapcanvas(city_tile(pcity));
        break;
      }
    }  
  }

}

/**************************************************************************
 Enables and disables the correct units commands for the unit in focus.
**************************************************************************/
function update_unit_order_commands()
{
  var unit_actions = { };
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    var ptype = unit_type(punit);
    var ptile = index_to_tile(punit['tile']);
    if (ptile == null) continue;
    var pcity = tile_city(ptile);

    if (pcity != null) {
      unit_actions["show_city"] = {name: "Show city"};
    }

  }

  unit_actions = $.extend(unit_actions, {
            "goto": {name: "Unit goto (G)"},
	    "tile_info": {name: "Tile info"}
            });

  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    var ptype = unit_type(punit);
    var ptile = index_to_tile(punit['tile']);
    if (ptile == null) continue;
    var pcity = tile_city(ptile);

    if (ptype['name'] == "Settlers") {
      $("#order_build_city").show();
      unit_actions["build"] = {name: "Build city (B)"}
    } else {
      $("#order_build_city").hide();
    }

    if (ptype['name'] == "Settlers" || ptype['name'] == "Workers" 
        || ptype['name'] == "Engineers") {
      if (!tile_has_extra(ptile, ROAD_ROAD)) {
        $("#order_road").show();
        $("#order_railroad").hide();
	unit_actions["road"] = {name: "Build road (R)"};
      } else if (player_invention_state(client.conn.playing, 65) == TECH_KNOWN
                 && tile_has_extra(ptile, ROAD_ROAD) 
               && !tile_has_extra(ptile, ROAD_RAIL)) {
        $("#order_road").hide();
        $("#order_railroad").show();
	unit_actions['railroad'] = {name: "Build railroad (R)"};
      } else {
        $("#order_road").hide();
        $("#order_railroad").hide();
      }
      $("#order_mine").show();
      $("#order_fortify").hide();
      $("#order_sentry").hide();
      $("#order_explore").hide();
      $("#order_auto_settlers").show();
      $("#order_pollution").show();
      unit_actions["mine"] = {name: "Build mine (M)"};
      unit_actions["autosettlers"] = {name: "Auto settler (A)"};

      if (tile_has_extra(ptile, EXTRA_FALLOUT)) {
        unit_actions["fallout"] = {name: "Remove fallout (N)"};
      }

      if (tile_has_extra(ptile, EXTRA_POLLUTION)) {
        $("#order_pollution").show();
	unit_actions["pollution"] = {name: "Remove pollution (P)"};
      } else {
        $("#order_pollution").hide();
      }

      if (tile_terrain(ptile)['name'] == "Forest") {
        $("#order_forest_remove").show(); 
        $("#order_irrigate").hide();
	unit_actions["forest"] = {name: "Cut down forest (I)"};
      } else if (!tile_has_extra(ptile, EXTRA_IRRIGATION)) {
        $("#order_irrigate").show();
        $("#order_forest_remove").hide(); 
        unit_actions["irrigation"] = {name: "Irrigation (I)"};	
      } else {
        $("#order_forest_remove").hide(); 	
        $("#order_irrigate").hide();
      }
      unit_actions["fortress"] = {name: "Build fortress (Shift-F)"};
      unit_actions["airbase"] = {name: "Build Airbase (E)"};

    } else {
      $("#order_road").hide();
      $("#order_railroad").hide();
      $("#order_mine").hide();
      $("#order_irrigate").hide();
      $("#order_fortify").show();
      $("#order_auto_settlers").hide();
      $("#order_sentry").show();
      $("#order_explore").show();
      $("#order_pollution").hide();

    }

    if (ptype['name'] == "Spy" || ptype['name'] == "Diplomat") {
      unit_actions["action_selection"] = {name: "Diplomatic action (D)"};
    }

    if (ptype['name'] == "Engineers") {
      $("#order_transform").show();
      unit_actions["transform"] = {name: "Transform terrain (O)"};
    } else {
      $("#order_transform").hide();
    }

    if (ptype['name'] == "Nuclear") {
      $("#order_nuke").show();
      unit_actions["nuke"] = {name: "Detonate Nuke (Shift-N)"};
    } else {
      $("#order_nuke").hide();
    }

    if (ptype['name'] == "Paratroopers") {
      $("#order_paradrop").show();
      unit_actions["paradrop"] = {name: "Paradrop"};
    } else {
      $("#order_paradrop").hide();
    }

    if (ptype['attack_strength'] > 0) {
      $("#order_pillage").show();
      unit_actions["pillage"] = {name: "Pillage (Shift-P)"};
    } else {
      $("#order_pillage").hide();
    }

    if (pcity == null || punit['homecity'] == 0 || (pcity != null && punit['homecity'] == pcity['id'])) {
      $("#order_change_homecity").hide();
    } else if (pcity != null && punit['homecity'] != pcity['id']) {
      $("#order_change_homecity").show();
      unit_actions["homecity"] = {name: "Change homecity of unit (H)"};
    }

    if (pcity != null) {
      unit_actions["airlift"] = {name: "Airlift (Shift-L)"};
    }

  }
  unit_actions = $.extend(unit_actions, {
            "explore": {name: "Auto explore (X)"},
            "fortify": {name: "Fortify (F)"},
            "sentry": {name: "Sentry (S)"},
            "wait": {name: "Wait (W)"},
            "noorders": {name: "No orders (J)"},
            "upgrade": {name: "Upgrade unit (U)"},
            "disband": {name: "Disband (Shift-D)"}
            });

  if (is_touch_device()) {
    $(".context-menu-list").css("width", "600px");
    $(".context-menu-item").css("font-size", "220%");
  }
  $(".context-menu-list").css("z-index", 5000);

  return unit_actions;
}


/**************************************************************************
...
**************************************************************************/
function init_game_unit_panel()
{
  unitpanel_active = true;

  $("#game_unit_panel").attr("title", "Units");		
  $("#game_unit_panel").dialog({
			bgiframe: true,
			modal: false,
			width: "250px",
			height: "auto",
			resizable: false,
			dialogClass: 'unit_dialog',
			position: {my: 'right bottom', at: 'right bottom', of: "#canvas_div"},
			close: function(event, ui) { unitpanel_active = false;}

		});
	
  $("#game_unit_panel").dialog('open');		
  $("#ui-dialog-title-game_unit_panel").css("margin-top", "-5px");
  $("#game_unit_panel").parent().css("overflow", "hidden");		
}

/**************************************************************************
 Find the nearest available unit for focus, excluding any current unit
 in focus unless "accept_current" is TRUE.  If the current focus unit
 is the only possible unit, or if there is no possible unit, returns NULL.
**************************************************************************/
function find_best_focus_candidate(accept_current)
{
  if (client_is_observer()) return null;
 
  var sorted_units = [];
  for (unit_id in units) {
    var punit = units[unit_id];
    if (punit['owner'] == client.conn.playing.playerno) {
      sorted_units.push(punit);
    }
  } 
  sorted_units.sort(unit_distance_compare);

  for (var i = 0; i < sorted_units.length; i++) {
    var punit = sorted_units[i];
    if ((!unit_is_in_focus(punit) || accept_current)
       && punit['owner'] == client.conn.playing.playerno
       && punit['activity'] == ACTIVITY_IDLE
       && punit['movesleft'] > 0
       && punit['done_moving'] == false
       && punit['ai'] == false
       && waiting_units_list.indexOf(punit['id']) < 0
       && punit['transported'] == false) {
         return punit;
    }
  }

  /* check waiting units for focus candidates */
  for (var i = 0; i < waiting_units_list.length; i++) {
      var punit = game_find_unit_by_number(waiting_units_list[i]);
      if (punit != null && punit['movesleft'] > 0) {
        waiting_units_list.splice(i, 1);
        return punit;
      }
  }

  return null;
}

/**************************************************************************
...
**************************************************************************/
function unit_distance_compare(unit_a, unit_b)
{
  if (unit_a == null || unit_b == null) return 0;
  var ptile_a = index_to_tile(unit_a['tile']);
  var ptile_b = index_to_tile(unit_b['tile']);

  if (ptile_a['x'] == ptile_b['x'] && ptile_a['y'] == ptile_b['y']) {
    return 0;
  } else if (ptile_a['x'] > ptile_b['x'] || ptile_a['y'] > ptile_b['y']) {
    return 1;
  } else {
    return -1;
  }
}


/**************************************************************************
  Sets the focus unit directly.  The unit given will be given the
  focus; if NULL the focus will be cleared.

  This function is called for several reasons.  Sometimes a fast-focus
  happens immediately as a result of a client action.  Other times it
  happens because of a server-sent packet that wakes up a unit.
**************************************************************************/
function set_unit_focus(punit)
{
  current_focus = [];
  if (punit == null) {
    current_focus = [];
  } else {
    current_focus[0] = punit;
  }
  update_unit_info_label(current_focus);
  update_unit_order_commands();
}

/**************************************************************************
 See set_unit_focus()
**************************************************************************/
function set_unit_focus_and_redraw(punit)
{
  current_focus = [];
  
  if (punit == null) {
    current_focus = [];
  } else {
    current_focus[0] = punit;
  }

  auto_center_on_focus_unit(); 
  update_unit_info_label(current_focus);
  update_unit_order_commands();
  $("#game_unit_orders_default").show();

}

/**************************************************************************
 ...
**************************************************************************/
function set_unit_focus_and_activate(punit)
{
  set_unit_focus_and_redraw(punit);
  request_new_unit_activity(punit, ACTIVITY_IDLE, EXTRA_NONE);

}

/**************************************************************************
 See set_unit_focus_and_redraw()
**************************************************************************/
function city_dialog_activate_unit(punit)
{
  request_new_unit_activity(punit, ACTIVITY_IDLE, EXTRA_NONE);
  close_city_dialog();
  set_unit_focus_and_redraw(punit);
}

/**************************************************************************
Center on the focus unit, if off-screen and auto_center_on_unit is true.
**************************************************************************/
function auto_center_on_focus_unit()
{
  if (active_city != null) return; /* don't change focus while city dialog is active.*/

  var ptile = find_a_focus_unit_tile_to_center_on();
  
  if (ptile != null && auto_center_on_unit) {
    center_tile_mapcanvas(ptile);
  }
}

/****************************************************************************
  Finds a single focus unit that we can center on.  May return NULL.
****************************************************************************/
function find_a_focus_unit_tile_to_center_on()
{
  var funit = current_focus[0];
  
  if (funit == null) return null;
  
  return index_to_tile(funit['tile']);
}

/**************************************************************************
Return a pointer to a visible unit, if there is one.
**************************************************************************/
function find_visible_unit(ptile)
{
  var panyowned = null;
  var panyother = null;
  var ptptother = null;
  var pfocus = null;

  /* If no units here, return nothing. */
  if (unit_list_size(tile_units(ptile))==0) {
    return null;
  }
  
  /* If the unit in focus is at this tile, show that on top */
  var pfocus = get_focus_unit_on_tile(ptile);
  if (pfocus != null) {
    return pfocus;
  }
  
  /* If a city is here, return nothing (unit hidden by city). */
  if (tile_city(ptile) != null) {
    return null;
  }
  
  /* TODO: add missing C logic here.*/
  var vunits = tile_units(ptile);
  for (var i = 0; i < vunits.length; i++) {
    var aunit = vunits[i];
    if (aunit['anim_list'] != null && aunit['anim_list'].length > 0) {
      return aunit;
    }
  }

  for (var i = 0; i < vunits.length; i++) {
    var tunit = vunits[i];
    if (tunit['transported'] == false) {
      return tunit;
    } 
  }   

  return tile_units(ptile)[0];
}

/**********************************************************************
TODO: not complete yet
***********************************************************************/
function get_drawable_unit(ptile, citymode)
{
  var punit = find_visible_unit(ptile);

  if (punit == null) return null;

  /*if (citymode != null && unit_owner(punit) == city_owner(citymode))
    return null;*/
  
  if (!unit_is_in_focus(punit) || current_focus.length > 0 ) {
    return punit;
  } else {
    return null;
  }
}

/**************************************************************************
 Handles everything when the user clicked a tile
**************************************************************************/
function do_map_click(ptile, qtype)
{
  if (ptile == null || client_is_observer()) return;
 
  if (current_focus.length > 0 && current_focus[0]['tile'] == ptile['index']) {
    /* clicked on unit at the same tile, then deactivate goto and show context menu. */
    deactivate_goto();
    $("#canvas").contextMenu();
    return;
  }
  var sunits = tile_units(ptile);
  var pcity = tile_city(ptile);

  if (goto_active) {
    /* Get the path the server sent using PACKET_GOTO_PATH. */
    var goto_path = goto_request_map[ptile['x'] + "," + ptile['y']];

    if (current_focus.length > 0
        /* there is a path for this tile */
        && goto_path != null
        /* the path for this tile has been received */
        && goto_path != true) {
      var punit = current_focus[0];

      /* The tile the unit currently is standing on. */
      var old_tile = index_to_tile(punit['tile']);

      /* Create an order to move along the path. */
      var packet = {
        "pid"      : packet_unit_orders,
        "unit_id"  : punit['id'],
        "src_tile" : old_tile['index'],
        "length"   : goto_path['length'],
        "repeat"   : false,
        "vigilant" : false,

        /* Each individual order is added later. */

        "dest_tile": ptile['index']
      };

      /* Add each individual order. */
      packet['orders'] = [];
      packet['dir'] = [];
      packet['activity'] = [];
      packet['target'] = [];
      for (var i = 0; i < goto_path['length']; i++) {
        /* TODO: Have the server send the full orders in stead of just the
         * dir part. Use that data in stead. */
        if (i + 1 != goto_path['length']) {
          /* Don't try to do an action in the middle of the path. */
          packet['orders'][i] = ORDER_MOVE;
        } else {
          /* It is OK to end the path in an action. */
          packet['orders'][i] = ORDER_ACTION_MOVE;
        }

        packet['dir'][i] = goto_path['dir'][i];
        packet['activity'][i] = ACTIVITY_LAST;
        packet['target'][i] = EXTRA_NONE;
      }

      /* Send the order to move using the orders system. */
      send_request(JSON.stringify(packet));
    }
  
    deactivate_goto();
    update_unit_focus();

  } else if (paradrop_active && current_focus.length > 0) {
    var punit = current_focus[0];
    var packet = {"pid" : packet_unit_paradrop_to, "unit_id" : punit['id'], "tile": ptile['index'] };
    send_request(JSON.stringify(packet));
    paradrop_active = false;

  } else if (airlift_active && current_focus.length > 0) {
    var punit = current_focus[0];
    var pcity = tile_city(ptile);
    if (pcity != null) {
      var packet = {"pid" : packet_unit_airlift, "unit_id" : punit['id'], "city_id": pcity['id'] };
      send_request(JSON.stringify(packet));
    }
    airlift_active = false;

  } else if (current_focus.length > 0 && sunits != null && sunits.length == 0 
             && pcity == null && !is_touch_device()) {
    activate_goto();

  } else {
    if (pcity != null) {
      if (pcity['owner'] == client.conn.playing.playerno) {
	if (sunits != null && sunits.length > 0 
            && sunits[0]['activity'] == ACTIVITY_IDLE) {
          set_unit_focus_and_redraw(sunits[0]);
          $("#canvas").contextMenu();
	} else {
          show_city_dialog(pcity);
	}
      }
      return;
    }
    
    if (sunits != null && sunits.length == 0) {
      /* Clicked on a tile with no units. */
      set_unit_focus_and_redraw(null);
    } else if (sunits != null && sunits.length > 0 ) {
      if (sunits[0]['owner'] == client.conn.playing.playerno) {
         if (sunits.length == 1) {
          /* A single unit has been clicked with the mouse. */
          var unit = sunits[0];  
	  set_unit_focus_and_activate(unit);
        } else {
          /* more than one unit is on the selected tile. */
          set_unit_focus_and_redraw(sunits[0]);
          update_select_unit_dialog(sunits);
        } 

        if (is_touch_device()) {
          $("#canvas").contextMenu();
	}
      }
    }
    
  }

  paradrop_active = false;
  airlift_active = false;
   
}

/**************************************************************************
 Callback to handle keyboard events
**************************************************************************/
function keyboard_listener(ev)
{
  // Check if focus is in chat field, where these keyboard events are ignored.
  if (!keyboard_input) return;

  if (!ev) ev = window.event;
  var keyboard_key = String.fromCharCode(ev.keyCode);

  civclient_handle_key(keyboard_key, ev.keyCode, ev['ctrlKey'],  ev['altKey'], ev['shiftKey']  );

  

}

/**************************************************************************
 Handles everything when the user typed on the keyboard.
**************************************************************************/
function
civclient_handle_key(keyboard_key, key_code, ctrl, alt, shift)
{

  switch (keyboard_key) {
    case 'B':
      request_unit_build_city();
 
      break;
      
    case 'G':
      if (current_focus.length > 0) {
        activate_goto();
      }
    break;

    case 'H':
      key_unit_homecity();
    break;
      
    case 'X':
      key_unit_auto_explore();
    break;  
    
    case 'A':
      key_unit_auto_settle();
    break;

    case 'L':
      if (shift) {
        key_unit_airlift();
      }
    break;

    case 'W':
      key_unit_wait();
    break;

    case 'J':
      key_unit_noorders();
    break;
     
    case 'R':
      key_unit_road();
    break;

    case 'E':
      if (shift) {
        key_unit_airbase();
      }
    break;

    case 'F':
      if (shift) {
        key_unit_fortress();
      } else {
        key_unit_fortify();
      }
    break;

    case 'I':
      key_unit_irrigate();
    break;      

   case 'U':
      key_unit_upgrade();
    break;  

    case 'S':
      key_unit_sentry();
    break;         
    case 'P':
      if (shift) {
        key_unit_pillage();
      } else {
        key_unit_pollution();
      }
    break;
    
    case 'M':
      key_unit_mine();
    break;

    case 'O':
      key_unit_transform();
    break; 

    case 'C':
      if (ctrl) {
        show_citybar = !show_citybar;
      }
    break; 


    case 'N':
      if (shift) {
        key_unit_nuke();
      } else {
        key_unit_fallout();
      }
    break;        

    case 'Q':
      if (alt) civclient_benchmark(0); 
    break;

    case 'D':
      if (shift) {
        key_unit_disband();
      } else if (alt || ctrl) {
        show_debug_info();
      } else {
        key_unit_action_select();
      }
    break;

  };
  
  switch (key_code) {
    case 13:
      if (shift) send_end_turn();
      break;

    case 35: //1
    case 97:
      key_unit_move(DIR8_SOUTH);
    break;
  
    case 40: // 2
    case 98:
      key_unit_move(DIR8_SOUTHEAST);  
      break;

    case 34: // 3
    case 99:
      key_unit_move(DIR8_EAST);
      break;

    case 37: // 4
    case 100:
      key_unit_move(DIR8_SOUTHWEST);
      break;

    case 39: // 6
    case 102:
      key_unit_move(DIR8_NORTHEAST);
      break;

    case 36: // 7
    case 103:
      key_unit_move(DIR8_WEST);
      break;

    case 38: // 8
    case 104:
      key_unit_move(DIR8_NORTHWEST);
      break;

    case 33: // 9
    case 105:
      key_unit_move(DIR8_NORTH);
      break;
  
  
  };
  
}

/**************************************************************************
 Handles everything when the user clicked on the context menu
**************************************************************************/
function handle_context_menu_callback(key)
{
  switch (key) {
    case "build":
      request_unit_build_city();
      break;

    case "tile_info":
      var ptile = find_a_focus_unit_tile_to_center_on();
      if (ptile != null) popit_req(ptile);
      break;

    case "goto":
      activate_goto();
      break;

    case "explore":
      key_unit_auto_explore();
      break;

    case "fortify":
      key_unit_fortify();
      break;

    case "road":
      key_unit_road();
      break;

    case "railroad":
      key_unit_road();
      break;

    case "mine":
      key_unit_mine();
      break;

    case "autosettlers":
      key_unit_auto_settle()
      break;

    case "fallout":
      key_unit_fallout();
      break;

    case "pollution":
      key_unit_pollution();
      break;

    case "forest":
      key_unit_irrigate();
      break;

    case "irrigation":
      key_unit_irrigate();
      break;

    case "fortress":
      key_unit_fortress();
      break;

    case "airbase":
      key_unit_airbase();
      break;

    case "transform":
      key_unit_transform();
      break;

    case "nuke":
      key_unit_nuke();
      break;

    case "paradrop":
      key_unit_paradrop();
      break;

    case "pillage":
      key_unit_pillage();
      break;

    case "homecity":
      key_unit_homecity();
      break;

    case "airlift":
      key_unit_airlift();
      break;

    case "sentry":
      key_unit_sentry();
      break;

    case "wait": 
      key_unit_wait();
      break;

    case "noorders": 
      key_unit_noorders();
      break;

    case "upgrade":
      key_unit_upgrade();
      break;

    case "disband":
      key_unit_disband();
      break;

    case "action_selection":
      key_unit_action_select(); 
      break;

    case "show_city":
      var ptile = find_a_focus_unit_tile_to_center_on();
      if (ptile != null) {
        var pcity = tile_city(ptile)
        show_city_dialog(pcity);
      }
      break;
  };
  if (key != "goto" && is_touch_device()) {
    deactivate_goto();
  }
}

/**************************************************************************
 ...
**************************************************************************/
function activate_goto()
{
  goto_active = true;
  mapview_canvas.style.cursor = "crosshair";

  if (current_focus.length > 0) {
    if (intro_click_description) {
      if (is_touch_device()) {
        add_chatbox_text("Carefully drag unit to the tile you want it to go to.");
      } else {
        add_chatbox_text("Click on the tile to send this unit to.");
      }
      intro_click_description = false;
    }

  } else {
    add_chatbox_text("First select a unit to move by clicking on it, then click on the"
		     + " goto button or the 'G' key, then click on the position to move to.");
    deactivate_goto();
  }

}

/**************************************************************************
 ...
**************************************************************************/
function deactivate_goto()
{
  goto_active = false;
  mapview_canvas.style.cursor = "default";
  goto_request_map = {};
  goto_turns_request_map = {};
  clear_goto_tiles();

  // update focus to next unit after 600ms.
  setTimeout("update_unit_focus();", 600);


}

/**************************************************************************
 Ends the current turn.
**************************************************************************/
function send_end_turn()
{
  $("#turn_done_button").button( "option", "disabled", true); 
  if (!is_touch_device()) $("#turn_done_button").tooltip({ disabled: true });
  var packet = {"pid" : packet_player_phase_done, "turn" : game_info['turn']};
  send_request(JSON.stringify(packet));
}


/**************************************************************************
 Tell the units in focus to auto explore.
**************************************************************************/
function key_unit_auto_explore()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    request_new_unit_activity(punit, ACTIVITY_EXPLORE, EXTRA_NONE);
  }
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Tell the unit to wait (focus to next unit with moves left)
**************************************************************************/
function key_unit_wait()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    waiting_units_list.push(punit['id']);
  }
  advance_unit_focus();
}

/**************************************************************************
 Tell the unit to have no orders this turn, set unit to done moving.
**************************************************************************/
function key_unit_noorders()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    punit['done_moving'] = true;
  }

  advance_unit_focus();
}


/**************************************************************************
 Tell the units in focus to sentry.
**************************************************************************/
function key_unit_sentry()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    request_new_unit_activity(punit, ACTIVITY_SENTRY, EXTRA_NONE);
  }
  setTimeout(update_unit_focus, 700); 
}

/**************************************************************************
 Tell the units in focus to fortify.
**************************************************************************/
function key_unit_fortify()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    request_new_unit_activity(punit, ACTIVITY_FORTIFYING, EXTRA_NONE);
  }
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Tell the units in focus to build base.
**************************************************************************/
function key_unit_fortress()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i];
    /* EXTRA_NONE -> server decides */
    request_new_unit_activity(punit, ACTIVITY_BASE, EXTRA_NONE);
  }
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Tell the units in focus to build airbase.
**************************************************************************/
function key_unit_airbase()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i];
    request_new_unit_activity(punit, ACTIVITY_BASE, BASE_AIRBASE);
  }
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Tell the units in focus to irrigate.
**************************************************************************/
function key_unit_irrigate()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i];
    /* EXTRA_NONE -> server decides */
    request_new_unit_activity(punit, ACTIVITY_IRRIGATE, EXTRA_NONE);
  }
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Tell the units to remove pollution.
**************************************************************************/
function key_unit_pollution()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    request_new_unit_activity(punit, ACTIVITY_POLLUTION, EXTRA_NONE);
  }  
  setTimeout(update_unit_focus, 700);
}


/**************************************************************************
 Tell the units to explode as a nuke.
**************************************************************************/
function key_unit_nuke()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    var packet = {"pid" : packet_unit_nuke, "unit_id" : punit['id']};
      send_request(JSON.stringify(packet));  
  }  
  update_unit_focus();
}

/**************************************************************************
 Tell the units to upgrade.
**************************************************************************/
function key_unit_upgrade()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    var packet = {"pid" : packet_unit_upgrade, "unit_id" : punit['id']};
      send_request(JSON.stringify(packet));  
  }  
  update_unit_focus();
}

/**************************************************************************
 Tell the units to paradrop.
**************************************************************************/
function key_unit_paradrop()
{
  paradrop_active = true;
  add_chatbox_text("Click on the tile to send this paratrooper to.");
}

/**************************************************************************
 Tell the units to airlift.
**************************************************************************/
function key_unit_airlift()
{
  airlift_active = true;
  add_chatbox_text("Click on the city to airlift this unit to.");
}

/**************************************************************************
 Tell the units to remove nuclear fallout.
**************************************************************************/
function key_unit_fallout()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    request_new_unit_activity(punit, ACTIVITY_FALLOUT, EXTRA_NONE);
  }  
  update_unit_focus();
}

/**************************************************************************
 Tell the units to transform the terrain.
**************************************************************************/
function key_unit_transform()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    request_new_unit_activity(punit, ACTIVITY_TRANSFORM, EXTRA_NONE);
  }  
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Tell the units in focus to pillage.
**************************************************************************/
function key_unit_pillage()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    request_new_unit_activity(punit, ACTIVITY_PILLAGE, EXTRA_NONE);
  }  
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Tell the units in focus to mine.
**************************************************************************/
function key_unit_mine()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i];
    /* EXTRA_NONE -> server decides */
    request_new_unit_activity(punit, ACTIVITY_MINE, EXTRA_NONE);
  }
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Tell the units in focus to build road or railroad.  
**************************************************************************/
function key_unit_road()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    var ptile = index_to_tile(punit['tile']);
    if (!tile_has_extra(ptile, ROAD_ROAD)) {
      request_new_unit_activity(punit, ACTIVITY_GEN_ROAD, extras['Road']['id']);
    } else if (!tile_has_extra(ptile, ROAD_RAIL)) {
      request_new_unit_activity(punit, ACTIVITY_GEN_ROAD, extras['Railroad']['id']);
    }
  }
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Changes unit homecity to the city on same tile.
**************************************************************************/
function key_unit_homecity()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    var ptile = index_to_tile(punit['tile']);
    var pcity = tile_city(ptile);

    if (pcity != null) {
      var packet = {"pid" : packet_unit_change_homecity, "unit_id" : punit['id'],
                "city_id" : pcity['id']};
      send_request(JSON.stringify(packet));
      $("#order_change_homecity").hide();
    }

  }
}

/**************************************************************************
  Show action selection dialog for unit(s).
**************************************************************************/
function key_unit_action_select()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i];
    process_diplomat_arrival(punit, punit['tile']);
  }
}

/**************************************************************************
  Call to request (from the server) that the focus unit is put into
  autosettler mode.
**************************************************************************/
function key_unit_auto_settle()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    request_unit_autosettlers(punit);
  }  
  setTimeout(update_unit_focus, 700);
}



/**************************************************************************
 ...
**************************************************************************/
function request_new_unit_activity(punit, activity, target)
{

  var packet = {"pid" : packet_unit_change_activity, "unit_id" : punit['id'],
                "activity" : activity, "target" : target };
  send_request(JSON.stringify(packet));
}


/****************************************************************************
  Call to request (from the server) that the settler unit is put into
  autosettler mode.
****************************************************************************/
function request_unit_autosettlers(punit)
{
  if (punit != null ) {
    var packet = {"pid" : packet_unit_autosettlers, "unit_id" : punit['id']};
    send_request(JSON.stringify(packet));
  }
}


/****************************************************************************
  Request that a city is built.
****************************************************************************/
function request_unit_build_city()
{
  if (current_focus.length > 0) {
    var punit = current_focus[0];
    if (punit != null) { 

      if (punit['movesleft'] == 0) {
        add_chatbox_text("Unit has no moves left to build city");
        return;
      }

      var ptype = unit_type(punit);
      if (ptype['name'] == "Settlers" || ptype['name'] == "Engineers") {
        var packet = {"pid" : packet_city_name_suggestion_req, "unit_id" : punit['id'] };
        send_request(JSON.stringify(packet));
      }
    }
  }
}

/**************************************************************************
 Tell the units in focus to disband.
**************************************************************************/
function key_unit_disband()
{
  var funits = get_units_in_focus();
  for (var i = 0; i < funits.length; i++) {
    var punit = funits[i]; 
    var packet = {"pid" : packet_unit_disband, "unit_id" : punit['id'] };
    send_request(JSON.stringify(packet));

    /* Also remove unit immediately in client, to ensure it is removed. */
    clear_tile_unit(punit);
    client_remove_unit(punit);

  }  
  setTimeout(update_unit_focus, 700);
}

/**************************************************************************
 Moved the unit in focus in the specified direction.
**************************************************************************/
function key_unit_move(dir) 
{
  if (current_focus.length > 0) {
    var punit = current_focus[0];
    if (punit == null) {
      return;
    }
    
    var ptile = index_to_tile(punit['tile']);
    if (ptile == null) {
      return;
    }

    var newtile = mapstep(ptile, dir);
    if (newtile == null) {
      return;
    }

    /* Send the order to move using the orders system. */
    var packet = {
      "pid"      : packet_unit_orders,
      "unit_id"  : punit['id'],
      "src_tile" : ptile['index'],
      "length"   : 1,
      "repeat"   : false,
      "vigilant" : false,
      "orders"   : [ORDER_ACTION_MOVE],
      "dir"      : [dir],
      "activity" : [ACTIVITY_LAST],
      "target"   : [EXTRA_NONE],
      "dest_tile": newtile['index']
    };

    send_request(JSON.stringify(packet));
  }

  deactivate_goto();
}

/**************************************************************************
 ...
**************************************************************************/
function process_diplomat_arrival(pdiplomat, target_tile_id)
{
  var ptile = index_to_tile(target_tile_id);

  /* No queue. */

  if (pdiplomat != null && ptile != null) {
    var packet = {
      "pid" : packet_unit_get_actions,
      "actor_unit_id" : pdiplomat['id'],
      "target_unit_id" : IDENTITY_NUMBER_ZERO,
      "target_city_id" : IDENTITY_NUMBER_ZERO,
      "target_tile_id": target_tile_id,
      "disturb_player": true
    };
    send_request(JSON.stringify(packet));
  }
}

/****************************************************************************
  Request GOTO path for unit with unit_id, and dst_x, dst_y in map coords.
****************************************************************************/
function request_goto_path(unit_id, dst_x, dst_y)
{
  if (goto_request_map[dst_x + "," + dst_y] == null) {
    goto_request_map[dst_x + "," + dst_y] = true;

    var packet = {"pid" : packet_goto_path_req, "unit_id" : unit_id,
                  "goal" : map_pos_to_tile(dst_x, dst_y)['index']};
    send_request(JSON.stringify(packet));
    clear_goto_tiles(); 
    current_goto_turns = null;
    $("#unit_text_details").html("Choose unit goto");
    setTimeout(update_mouse_cursor, 700);
  } else {
    update_goto_path(goto_request_map[dst_x + "," + dst_y]);
  }
}

/****************************************************************************
...
****************************************************************************/
function check_request_goto_path()
{
 if (goto_active && current_focus.length > 0 
      && prev_mouse_x == mouse_x && prev_mouse_y == mouse_y) {
    var ptile = canvas_pos_to_tile(mouse_x, mouse_y);
    if (ptile != null) {
      /* Send request for goto_path to server. */
      request_goto_path(current_focus[0]['id'], ptile['x'], ptile['y']);
    }
  }
  prev_mouse_x = mouse_x;
  prev_mouse_y = mouse_y;

}

/****************************************************************************
  Show the GOTO path in the unit_goto_path packet.
****************************************************************************/
function update_goto_path(goto_packet)
{
  clear_goto_tiles();
  var punit = units[goto_packet['unit_id']];
  if (punit == null) return;
  var t0 =  index_to_tile(punit['tile']);
  var ptile = t0;
  var goaltile = index_to_tile(goto_packet['dest']);

  for (var i = 0; i < goto_packet['dir'].length; i++) {
    if (ptile == null) break;
    var dir = goto_packet['dir'][i];
    ptile['goto_dir'] = dir; 
    ptile = mapstep(ptile, dir);
  }

  current_goto_turns = goto_packet['turns'];

  goto_request_map[goaltile['x'] + "," + goaltile['y']] = goto_packet;
  goto_turns_request_map[goaltile['x'] + "," + goaltile['y']] 
	  = current_goto_turns;

  if (current_goto_turns != undefined) {
    $("#unit_text_details").html("Turns for goto: " + current_goto_turns);
  } 
  update_mouse_cursor();
}

